#!/usr/bin/python
# test.py - this script will test the basic functionalities of the 8-puzzle implementation

import sys,os,random
from imp import find_module, new_module, PY_SOURCE
from searchP3 import Node

# user script should have the following modules (functions)
EXPECTED = ( "goalp", "successors", "edgecost", "hMisplaced", "hManhattan", "hPatternDB")

# a module that import the user script
def import_script(name):
    fileobj, path, description = find_module(name)

    if description[2] != PY_SOURCE:
        raise ImportError("no source file found")

    code = compile(fileobj.read(), path, "exec")

    expected = list(EXPECTED)
    for const in code.co_consts:
        if isinstance(const, type(code)) and const.co_name in expected:
            expected.remove(const.co_name)
    if expected:
        raise ImportError("missing expected function: {}".format(expected))

    module = new_module(name)
    exec(code, module.__dict__)
    sys.modules[name] = module
    return module

# module gen_rand_state(): returns a random board configuration -- will be used for grading
def gen_rand_state():
	while True:
		p = list(range(0,9))
		random.shuffle(p)
		if p != gen_goal_state():
			break
	return ((p[0],p[1],p[2]),(p[3],p[4],p[5]),(p[6],p[7],p[8]))

def gen_goal_state():
	return ((1,2,3),(8,0,4),(7,6,5))


# main script
if __name__ == '__main__':
	
	if len(sys.argv) < 2:
		print("Usage: python", sys.argv[0], "<your script name (without .py)>\n")
		sys.exit()
	
	# your script is imported in 'code'
	code = import_script(sys.argv[1])
	# number of tests on each testing station; for grading this will increase to 10+
	n_tests = 3
	
	# dump header
	os.system('clear')
	print("--------------------------------------------------------------------------------")
	print(" CS2710 Assignment #1 automatic grader - Input: ", sys.argv[1], ".py")
	print("--------------------------------------------------------------------------------\n")
	
	
	# testing goalp()
	print("=> Tesing goalp(Node)...")
	t = 1	# for indexing test#
	
	g = Node(gen_goal_state())
	if code.goalp(g):
		print("Test#",t,": goalp(", g.state, ") correctly returned true  --  GO")
	else:
		print("Test#",t,": goalp(", g.state, ") incorrectly returned false  --  NO GO")

	for t in range(t+1,n_tests):
		s = Node(gen_rand_state())
		if code.goalp(s):
			print("Test#",t,": goalp(", s.state, ") incorrectly returned true  --  NO GO")
		else:
			print("Test#",t,": goalp(", s.state, ") correctly returned false  --  GO")
		
	
	
	# testing successors(); for grading this block will go into a loop and repeat 10 times
	print("\n=> Testing successors(state)...")
	
	# for grading, a random 's' and corresponding 'ans' will be generated by TA's code
	s = Node(((7,3,2),(8,6,4),(1,0,5)))
	ans = [((7,3,2),(8,0,4),(1,6,5)), ((7,3,2),(8,6,4),(0,1,5)), ((7,3,2),(8,6,4),(1,5,0))]
	t = 1	# for indexing test#
	
	Sucs = code.successors(s)
	if len(Sucs) == len(ans):
		ans_used = [False] * len(ans)
		
		for suc in Sucs:
			for i in range(1,len(Sucs)):
				if ans_used[i] == False and ans[i] == suc:
					ans_used[i] = True
		
		is_fail = False
		for i in range(1,len(Sucs)):
			if ans_used[i] == False:
				print("Test#",t,": successors(", s.state, ") returned incorrect results  --  NO GO")
				is_fail = True
				break
				
		if is_fail == False:
			print("Test#",t,": successors(", s.state, ") returned correct results  --  GO")
	else:
		print("Test#",t,": successors(", s.state, ") return incorrect results  --  NO GO")

	# testing successors() - 2nd example (hardcoded for a demonstration purpose)
	t = 2	# for indexing test#
	s = Node(((4,7,8),(2,0,1),(5,3,6)))
	ans = [((4,0,8),(2,7,1),(5,3,6)), ((4,7,8),(0,2,1),(5,3,6)), ((4,7,8),(2,1,0),(5,3,6)), ((4,7,8),(2,3,1),(5,0,6))]
	Sucs = code.successors(s)
	if len(Sucs) == len(ans):
		ans_used = [False] * len(ans)
		for suc in Sucs:
			for i in range(1,len(Sucs)):
				if ans_used[i] == False and ans[i] == suc:
					ans_used[i] = True
		
		is_fail = False
		for i in range(1,len(Sucs)):
			if ans_used[i] == False:
				print("Test#",t,": successors(", s.state, ") returned incorrect results  --  NO GO")
				is_fail = True
		if is_fail == False:
			print("Test#",t,": successors(", s.state, ") returned correct results  --  GO")		
	else:
		print("Test#",t,": successors(", s.state, ") return incorrect results  --  NO GO")
	
	
	
	# testing edgecost()
	print("\n=> Testing edgecost(state1,state2)...")
	# generating testing nodes; board1,2,3 are hardcoded here for a demonstration purpose
	board0 = Node(((1,2,3),(4,5,6),(7,8,0)))
	board1 = Node(((1,2,3),(4,5,6),(7,0,8)))
	board2 = Node(((1,2,3),(4,5,0),(7,8,6)))
	print("board0: ", board0.state)
	print("board1: ", board1.state)
	print("board2: ", board2.state)
	
	t = 1	# for indexing test#
	if code.edgecost(board0.state,board1.state) == 1:
		print("Test#",t,": edgecost(board0.state,board1.state) returned correct edge cost  --  GO")
	else:
		print("Test#",t,": edgecost(board0.state,board1.state) returned incorrect edge cost  --  NO GO")

	t = 2	# for indexing test#
	if code.edgecost(board0.state,board2.state) == 1:
		print("Test#",t,": edgecost(board0.state,board2.state) returned correct edge cost  --  GO")
	else:
		print("Test#",t,": edgecost(board0.state,board2.state) returned incorrect edge cost  --  NO GO")
	


	# testing heuristics
	# 2 sample boards; for grading, 10 random boards will be generated and examine your heuristic
	board1 = Node(((1,3,4),(8,6,2),(7,0,5)))
	board2 = Node(((4,2,5),(3,0,6),(1,8,7)))
	
	# testing heuristic1()
	print("\n=> Testing #misplaced heuristic...")
	t = 1	# for indexing test#
	if code.hMisplaced(board1) == 4:	# h=4
		print("Test#",t,": hMisplaced(",board1.state,") returned correct estimation  --  GO")
	else:
		print("Test#",t,": hMisplaced(",board1.state,") returned incorrect estimation  --  NO GO")

	t = 2	# for indexing test#
	if code.hMisplaced(board2) == 7:	# h=7
		print("Test#",t,": hMisplaced(",board2.state,") returned correct estimation  --  GO")
	else:
		print("Test#",t,": hMisplaced(",board2.state,") returned incorrect estimation  --  NO GO")
	
	
	# testing heuristic2()
	print("\n=> Testing Manhattan distance heuristic...")
	t = 1	# for indexing test#
	if code.hManhattan(board1) == 5:	# h=5
		print("Test#",t,": hManhattan(",board1.state,") returned correct estimation  --  GO")
	else:
		print("Test#",t,": hManhattan(",board1.state,") returned incorrect estimation (yours =",code.hManhattan(board1),") --  NO GO")
	t = 2	# for indexing test#
	if code.hManhattan(board2) == 16:	#h=16
		print("Test#",t,": hManhattan(",board2.state,") returned correct estimation  --  GO")
	else:
		print("Test#",t,": hManhattan(",board2.state,") returned incorrect estimation (yours =",code.hManhattan(board2),")  --  NO GO")
	
	
	# testing heuristic3()
	print("\n=> Testing Pattern Database...")
	t = 1	# for indexing test#
	if code.hPatternDB(board1) <= 5:	# optimal = 5
		print("Test#",t,": hPatternDB(",board1.state,") is admissible  --  GO")
	else:
		print("Test#",t,": hPatternDB(",board1.state,") is not admissible  --  NO GO")
	t = 2	# for indexing test#
	if code.hPatternDB(board2) <= 18:	# optimal = 18
		print("Test#",t,": hPatternDB(",board2.state,") is admissible  --  GO")
	else:
		print("Test#",t,": hPatternDB(",board2.state,") is not admissible  --  NO GO")
	
	print("\n=> Executing the script...")
	print("...your script should run by a command like:\n\n  python eightPuzzle.py breadthfirst \"((1,2,3),(4,5,6),(7,8,0))\" treesearch\n")
	# Uncomment the following lines if you want to test your script
	#os.system("python " + sys.argv[1] + ".py breadthfirst "((1,2,3),(4,5,6),(7,8,0))" treesearch")
	#os.system("python " + sys.argv[1] + ".py depthfirst "((1,2,3),(4,5,6),(7,8,0))" treesearch")
	#os.system("python " + sys.argv[1] + ".py all "((1,2,3),(4,5,6),(7,8,0))" all")
	
	
	